#import "FireMethodFromLua.h"
#include <CoronaRuntime.h>

class FireMethodFromLua
{
public:
    typedef FireMethodFromLua Self;
    
public:
    static const char kName[];
    static const char kEvent[];
    
protected:
    FireMethodFromLua();
    
public:
    bool Initialize( CoronaLuaRef listener );
    
public:
    CoronaLuaRef GetListener() const { return fListener; }
    
public:
    static int Open( lua_State *L );
    
protected:
    static int Finalizer( lua_State *L );
    
public:
    static Self *ToLibrary( lua_State *L );
    
public:
    static int doSomeWork( lua_State *L );
    
private:
    CoronaLuaRef fListener;
    
};


const char FireMethodFromLua::kName[] = "FireMethodFromLua";
const char FireMethodFromLua::kEvent[] = "FireMethodFromLuaEvent";

FireMethodFromLua::FireMethodFromLua()
:    fListener( NULL )
{
}


bool
FireMethodFromLua::Initialize( CoronaLuaRef listener )
{
    // Can only initialize listener once
    bool result = ( NULL == fListener );
    
    if ( result )
    {
        fListener = listener;
    }
    
    return result;
}

int
FireMethodFromLua::Open( lua_State *L )
{
    // Register __gc callback
    const char kMetatableName[] = __FILE__; // Globally unique string to prevent collision
    CoronaLuaInitializeGCMetatable( L, kMetatableName, Finalizer );
    
    // Functions in library
    const luaL_Reg kVTable[] =
    {
        { "doSomeWork", doSomeWork },
        { NULL, NULL }
    };
    
    // Set library as upvalue for each library function
    Self *library = new Self;
    CoronaLuaPushUserdata( L, library, kMetatableName );
    
    luaL_openlib( L, kName, kVTable, 1 ); // leave "library" on top of stack
    
    return 1;
}

int
FireMethodFromLua::Finalizer( lua_State *L )
{
    Self *library = (Self *)CoronaLuaToUserdata( L, 1 );
    
    CoronaLuaDeleteRef( L, library->GetListener() );
    
    delete library;
    
    return 0;
}

FireMethodFromLua *
FireMethodFromLua::ToLibrary( lua_State *L )
{
    // library is pushed as part of the closure
    Self *library = (Self *)CoronaLuaToUserdata( L, lua_upvalueindex( 1 ) );
    return library;
}


// [Lua] FireMethodFromLua.doSomeWork()
int
FireMethodFromLua::doSomeWork( lua_State *L )
{
    return 1;
}

// ----------------------------------------------------------------------------

CORONA_EXPORT int luaopen_FireMethodFromLua( lua_State *L )
{
    return FireMethodFromLua::Open( L );
}
